# SourceMessagingService by David Bowland
# ./addons/eventscripts/sms/sms.py


# >>> To configure this addon please see sms.cfg (created when the addon is first loaded) <<<

"""
Sends SMS text message alerts to the cell phones of server admins.
Requires only ES 2.0.0.250i+
"""

"""
   SourceMessagingService allows other ESP addons to access its text messaging functionality.


   SourceMessagingService must be loaded to use its text messaging services:

import es
sms = es.import_addon('sms')


   You can then send an alert using:

sms.sendAlert(name, subject, text)

   name - alert name numbers must be registered for to receive the alert
   subject - subject of alert message
   text - text of alert message


   You can directly send a text message using:

sms.sendTxtMsg(number, message, carrier, subject="Server alert", sender=cvar_return_address)

   number - phone number to send the text message to
   message - message to send
   carrier - integer representation of the cell phone provider the number is registered with (see the list below)
   subject - subject of the text message
   sender - e-mail address from which the text message should originate (defaults to value of cvar "sms_return_address")
"""

"""
Carrier codes:

36 - Alltel PCS
41 - AT&T PCS
69 - Cingular Wireless
178 - SunCom
176 - Sprint PCS
182 - T-Mobile / Voicestream
203 - Verizon PCS
214 - Nextel

1 - MTS (Russia)
2 - PGSM (Hungary)
3 - Telecom Italia Mobile (Italy)
4 - Vodafone Omnitel (Italy)

5 - Max.Mobil (Austria)
6 - Orange (United Kingdom)
7 - Tele Danmark Mobil (Denmark)
8 - Sonofon (Denmark)
9 - Telia Mobitel (Sweden)
10 - Comviq GSM (Sweden)

11 - Europolitan (Sweden)
12 - Telenor Mobil (Norway)
13 - Netcom GSM (Norway)
14 - Plus GSM (Poland)
15 - D1 De TeMobil (Germany)
16 - D2 Mannesmann Mobilefunk (Germany)

17 - E-Plus (Germany)
18 - Celcom (Malaysia)
19 - Mobitel (Tanzania)
20 - Telecel (Portugal)
21 - Optimus (Portugal)
22 - TMN (Portugal)

23 - LMT (Latvia)
24 - UMC GSM (Ukraine)
25 - Kyivstar GSM (Ukraine)
26 - Si.Mobil (Slovenia)
27 - Mobitel (Slovenia)
28 - Eurotel (Czech Republic)

29 - Cellis & LibanCell (Lebanon)
30 - 3 River Wireless
31 - ACS Wireless
32 - Advantage Communications
33 - Airtouch Pagers
34 - AlphaNow

35 - Alltel
36 - Alltel PCS
37 - Ameritech Paging
38 - Ameritech Clearpath
39 - Andhra Pradesh Airtel
40 - Arch Pagers (PageNet)

41 - AT&T PCS
42 - AT&T Pocketnet PCS
43 - Beepwear
44 - BeeLine GSM
45 - Bell Atlantic

46 - Bell Canada
47 - Bell Mobility (Canada)
48 - Bell Mobility
49 - Bell South (Blackberry)
50 - Bell South
51 - Bell South Mobility

52 - Blue Sky Frog
53 - Bluegrass Cellular
54 - Boost
55 - BPL mobile
56 - Carolina Mobile Communications
57 - Cellular One East Coast

58 - Cellular One South West
59 - Cellular One PCS
60 - Cellular One
61 - Cellular One West
62 - Cellular South
63 - Central Vermont Communications

64 - CenturyTel
65 - Chennai RPG Cellular
66 - Chennai Skycell / Airtel
67 - Cincinnati Bell
68 - Cingular
69 - Cingular Wireless

70 - Clearnet
71 - Comcast
72 - Communication Specialists
73 - Comviq
74 - Cook Paging
75 - Corr Wireless Communications

215 - Cricket Wireless
76 - Delhi Aritel
77 - Digi-Page / Page Kansas
78 - Dobson Cellular Systems
79 - Dobson-Alex Wireless / Dobson-Cellular One
80 - DT T-Mobile

81 - Dutchtone / Orange-NL
82 - Edge Wireless
83 - EMT
84 - Escotel
85 - Fido
86 - Galaxy Corporation

87 - GCS Paging
88 - Goa BPLMobil
89 - Golden Telecom
90 - GrayLink / Porta-Phone
91 - GTE
92 - Gujarat Celforce

93 - Houston Cellular
94 - Infopage Systems
95 - Inland Cellular Telephone
96 - JSM Tele-Page
97 - Kerala Escotel
98 - Kolkata Airtel

99 - Kyivstar
100 - Lauttamus Communication
101 - LMT
102 - Maharashtra BPL Mobile
103 - Maharashtra Idea Cellular
104 - Manitoba Telecom Systems

105 - MCI Phone
106 - MCI Page
107 - Meteor
108 - Metrocall
109 - Metrocall 2-way
110 - Metro PCS

111 - Microcell
112 - Midwest Wireless
113 - MiWorld
114 - Mobilecom PA
115 - Mobilecomm
116 - Mobileone

117 - Mobilfone
118 - Mobility Bermuda
119 - Mobistar Belgium
120 - Mobitel Tanzania
121 - Mobtel Srbija
122 - Morris Wireless

123 - Motient
124 - Movistar
125 - Mumbai BPL Mobile
126 - Mumbai Orange
127 - NBTel
128 - Netcom

129 - NPI Wireless
130 - Ntelos
131 - O2
132 - O2 (M-mail)
133 - Omnipoint
134 - One Connect Austria

135 - OnlineBeep
136 - Optus Mobile
137 - Orange
138 - Orange Mumbai
139 - Orange - NL / Dutchtone
140 - Oskar

141 - P&T Luxembourg
142 - Pacific Bell
143 - PageMart
144 - PageMart Advanced /2way
145 - PageMart Canada

146 - PageNet Canada
147 - PageOne NorthWest
148 - PCS One
149 - Personal Communication
150 - Pioneer / Enid Cellular
151 - PlusGSM

152 - Pondicherry BPL Mobile
153 - Powertel
154 - Price Communications
155 - Primco
156 - Primtel
157 - ProPage

158 - Public Service Cellular
159 - Qualcomm
160 - Qwest
161 - RAM Page
162 - Rogers AT&T Wireless

163 - Rogers Canada
164 - Safaricom
165 - Satelindo GSM
166 - Satellink
167 - SBC Ameritech Paging
168 - SCS-900

169 - SFR France
170 - Skytel Pagers
171 - Simple Freedom
172 - Smart Telecom
173 - Southern LINC
174 - Southwestern Bell

175 - Sprint
176 - Sprint PCS
177 - ST Paging
178 - SunCom
179 - Sunrise Mobile
180 - Surewest Communicaitons

181 - Swisscom
182 - T-Mobile / Voicestream
183 - T-Mobile Austria
184 - T-Mobile Germany
185 - T-Mobile UK
186 - Tamil Nadu BPL Mobile

187 - Tele2 Latvia
188 - Telefonica Movistar
189 - Telenor
190 - Teletouch
191 - Telia Denmark
192 - Telus

193 - TIM
194 - Triton
195 - TSR Wireless
196 - UMC
197 - Unicel
198 - Uraltel

199 - US Cellular
200 - US West
201 - Uttar Pradesh Escotel
202 - Verizon Pagers
203 - Verizon PCS
204 - Vessotel

205 - Virgin Mobile
206 - Vodafone Italy
207 - Vodafone Japan
208 - Vodafone Spain
209 - Vodafone UK
210 - WebLink Wiereless

211 - West Central Wireless
212 - Western Wireless
213 - Wyndtell
214 - Nextel
"""

import cfglib
import cmdlib
import es
import langlib
import os.path
import playerlib
import services
import threading
import time
import urllib
from Queue import Queue
from usermsg import hudhint


import psyco
psyco.full()

###

info = es.AddonInfo()
info.name      = 'SourceMessagingService'
info.version   = '1'
info.basename  = 'sms'
info.url       = 'http://addons.eventscripts.com/addons/view/' + info.basename
info.author    = 'SuperDave'
info.poweredby = 'SMS service provided by: http://www.onlinetextmessage.com/'

###

THREAD_COUNT = 1 # Number of threads to create
MAX_MSG_LENGTH = 100 # Traditionally 160, this 100 character limit is imposed by the SMS relay

###

config = cfglib.AddonCFG(os.path.join(es.getAddonPath(info.basename), info.basename + '.cfg'))

config.text(info.name + ' release %s options' % info.version)
config.text('./addons/eventscripts/%(basename)s/%(basename)s.cfg' % {'basename': info.basename})
config.text(info.url)
config.text('')
config.text('Load this addon with: es_load ' + info.basename)
config.text('\n')
config.text('To configure language or TEXT MESSAGE CONTENT settings for %s please see %s_languages.ini' % (info.name, info.basename))
config.text('\n')
config.text('SMS service provided by: http://www.onlinetextmessage.com/')
config.text('\n')

config.text('***** Return e-mail ****')
config.text('Text messages are required to have a source so this option dictates the return e-mail.')
config.text('The e-mail address provided will receive replies from SourceMessagingService text messages')
config.text('but will also receive "thank you" e-mails from onlinetextmessage.com')
cvar_return_address = config.cvar('sms_return_address', '', 'Reply e-mail address for text messages')
config.text('\n')

config.text('***** On-demand alerts *****')
config.text('The name of on-demand alerts is "on_demand".')
cvar_demand_authorization = config.cvar('sms_demand_authorization', 'IDENTIFIED', 'Level of authorization players must have to use the on-demand SMS feature')
config.text('Possible values are: ROOT ADMIN POWERUSER IDENTIFIED')
config.text('For more information see the authorization FAQ: http://www.eventscripts.com/pages/Authorization_FAQ')
config.text('Players with the "sms_demand" capability will also be able to use the demand SMS feature')
config.text('If no authorization service is loaded all players with a valid Steam ID will be able to use the on-demand feature')
config.text('')
cvar_demand_command = config.cvar('sms_demand_command', '!alert', 'Say command to send a text message alert to admins')
config.text('')
cvar_demand_command_announce_chat = config.cvar('sms_demand_command_announce_chat', 2, '0 = no chat announcement, 1 = display say command in console only, 2 = display say command in chat area and console')
cvar_demand_command_announce_hud  = config.cvar('sms_demand_command_announce_hud',  0, '0 = no HUD announcement, 1 = display say command in a HUD hint')
config.text('')
cvar_demand_command_acknowledge_chat   = config.cvar('sms_demand_command_acknowledge_chat',    2, '0 = no chat announcement, 1 = display not authorized message in console only, 2 = display not authorized message in chat area and console')
cvar_demand_command_acknowledge_center = config.cvar('sms_demand_command_acknowledge_center',  1, '0 = no conter announcement, 1 = display not authorized message in a center message')
cvar_demand_command_acknowledge_hud    = config.cvar('sms_demand_command_acknowledge_hud',     0, '0 = no HUD announcement, 1 = display not authorized message in a HUD hint')
config.text('')
cvar_demand_command_noauth_chat   = config.cvar('sms_demand_command_noauth_chat',    2, '0 = no chat announcement, 1 = display not authorized message in console only, 2 = display not authorized message in chat area and console')
cvar_demand_command_noauth_center = config.cvar('sms_demand_command_noauth_center',  1, '0 = no conter announcement, 1 = display not authorized message in a center message')
cvar_demand_command_noauth_hud    = config.cvar('sms_demand_command_noauth_hud',     0, '0 = no HUD announcement, 1 = display not authorized message in a HUD hint')
config.text('\n')

config.text('***** Server restart alert *****')
config.text('The name of the restart alert is "server_restart".')
cvar_alert_restart = config.cvar('sms_alert_restart', 1, '0 = no alert on server restart, 1 = alert text message on server restart')
config.text('\n')

config.text('***** Ban alert *****')
config.text('Players registered to receive "server_addban" alerts will receive an alert when a player is banned.')
config.text('\n')

config.text('***** Cvar alerts *****')
config.text('All cvar alert names are "cvar_" follow by the name of the variable.')
config.text('')
config.text('Use the following server command to mark cvars to be watched by SMS:')
config.text('')
config.text('sms_cvar_watch <cvar>')
config.text('\n')

config.text('Place your sms_cvar_watch commands below:')
config.command('sms_cvar_watch')
config.text('\n')

config.text('Uncomment the following default options if you wish to use them:')
config.text('')
config.text('sms_cvar_watch rcon_password')
config.text('sms_cvar_watch sv_cheats')
config.text('sms_cvar_watch sv_password')
config.text('\n')

config.text('Use the following command to remove variables from the being watched by SMS:')
config.text('')
config.text('sms_cvar_unwatch <cvar>')
config.text('')
config.text('Due to the fact watched cvars can be specified above, this command will not be used by most users.')
config.text('\n')

config.text('***** Phone numbers *****')
config.text('')
config.text('Use the following server command to add phone numbers to SMS:')
config.text('')
config.text('sms_number_add <"phone number"> <carrier> ["Steam ID"]')
config.text('')
config.text('Dashes, spaces and parenthesis are optional. See the bottom of this file for a list of carrier codes.')
config.text('If provided, SMS will check the Steam ID corresponding with each number to see if the player is connected.')
config.text('If connected, the player will not receive a text message and will instead receive an ESC-menu.')
config.text('\n')

config.text('Place your sms_number_add commands below:')
config.command('sms_number_add')
config.text('\n')

config.text('Uncomment the following example options if you wish to use them:')
config.text('')
config.text('sms_number_add "(573) 555-1234" 41 "STEAM_0:0:00001"')
config.text('sms_number_add "(253) 555-1235" 176 "STEAM_0:0:00002"')
config.text('sms_number_add "(412) 555-1236" 203 "STEAM_0:0:00003"')
config.text('\n')

config.text('Use the following command to remove phone numbers from SMS:')
config.text('')
config.text('sms_number_remove <variable>')
config.text('')
config.text('Due to the fact phone numbers can be specified above, this command will not be used by most users.')
config.text('')

cvar_connected_display = config.cvar('sms_connected_display', 30, 'Number of seconds to display ESC text box to players in lieu of text messages, 0 = always send text message')
config.text('\n')

config.text('***** Alert groups *****')
config.text('')
config.text('Use the following server command to register phone numbers to receive alerts:')
config.text('')
config.text('sms_register_for_alert <"number"> <"alert name">')
config.text('')
config.text('Phone numbers will only receive alerts they are registered to receive.')
config.text('\n')

config.text('Place your sms_register_for_alert commands below:')
config.command('sms_register_for_alert')
config.text('\n')

config.text('Uncomment the following example options if you wish to use them:')
config.text('')
config.text('sms_register_for_alert "(573) 555-1234" "server_restart"')
config.text('sms_register_for_alert "(573) 555-1234" "on_demand"')
config.text('sms_register_for_alert "(573) 555-1234" "cvar_rcon_password"')
config.text('sms_register_for_alert "(573) 555-1234" "cvar_sv_cheats"')
config.text('sms_register_for_alert "(573) 555-1234" "cvar_sv_password"')
config.text('')
config.text('sms_register_for_alert "(253) 555-1235" "on_demand"')
config.text('sms_register_for_alert "(253) 555-1235" "cvar_sv_password"')
config.text('')
config.text('sms_register_for_alert "(412) 555-1236" "on_demand"')
config.text('sms_register_for_alert "(412) 555-1236" "cvar_sv_cheats"')
config.text('sms_register_for_alert "(412) 555-1236" "cvar_sv_password"')
config.text('\n')

config.text('Use the following command to unregister phone numbers for alerts:')
config.text('')
config.text('sms_unregister_for_alert <phone number> <"alert name">')
config.text('')
config.text('Due to the fact alerts can be specified above, this command will not be used by most users.')
config.text('\n')

config.text('***** Carrier codes *****')
config.text('Carrier codes are for reference only -- they are not options.')
config.text('')
config.text('36 - Alltel PCS')
config.text('41 - AT&T PCS')
config.text('69 - Cingular Wireless')
config.text('178 - SunCom')
config.text('176 - Sprint PCS')
config.text('182 - T-Mobile / Voicestream')
config.text('203 - Verizon PCS')
config.text('214 - Nextel')
config.text('')
config.text('1 - MTS (Russia)')
config.text('2 - PGSM (Hungary)')
config.text('3 - Telecom Italia Mobile (Italy)')
config.text('4 - Vodafone Omnitel (Italy)')
config.text('')
config.text('5 - Max.Mobil (Austria)')
config.text('6 - Orange (United Kingdom)')
config.text('7 - Tele Danmark Mobil (Denmark)')
config.text('8 - Sonofon (Denmark)')
config.text('9 - Telia Mobitel (Sweden)')
config.text('10 - Comviq GSM (Sweden)')
config.text('')
config.text('11 - Europolitan (Sweden)')
config.text('12 - Telenor Mobil (Norway)')
config.text('13 - Netcom GSM (Norway)')
config.text('14 - Plus GSM (Poland)')
config.text('15 - D1 De TeMobil (Germany)')
config.text('16 - D2 Mannesmann Mobilefunk (Germany)')
config.text('')
config.text('17 - E-Plus (Germany)')
config.text('18 - Celcom (Malaysia)')
config.text('19 - Mobitel (Tanzania)')
config.text('20 - Telecel (Portugal)')
config.text('21 - Optimus (Portugal)')
config.text('22 - TMN (Portugal)')
config.text('')
config.text('23 - LMT (Latvia)')
config.text('24 - UMC GSM (Ukraine)')
config.text('25 - Kyivstar GSM (Ukraine)')
config.text('26 - Si.Mobil (Slovenia)')
config.text('27 - Mobitel (Slovenia)')
config.text('28 - Eurotel (Czech Republic)')
config.text('')
config.text('29 - Cellis & LibanCell (Lebanon)')
config.text('30 - 3 River Wireless')
config.text('31 - ACS Wireless')
config.text('32 - Advantage Communications')
config.text('33 - Airtouch Pagers')
config.text('34 - AlphaNow')
config.text('')
config.text('35 - Alltel')
config.text('36 - Alltel PCS')
config.text('37 - Ameritech Paging')
config.text('38 - Ameritech Clearpath')
config.text('39 - Andhra Pradesh Airtel')
config.text('40 - Arch Pagers (PageNet)')
config.text('')
config.text('41 - AT&T PCS')
config.text('42 - AT&T Pocketnet PCS')
config.text('43 - Beepwear')
config.text('44 - BeeLine GSM')
config.text('45 - Bell Atlantic')
config.text('')
config.text('46 - Bell Canada')
config.text('47 - Bell Mobility (Canada)')
config.text('48 - Bell Mobility')
config.text('49 - Bell South (Blackberry)')
config.text('50 - Bell South')
config.text('51 - Bell South Mobility')
config.text('')
config.text('52 - Blue Sky Frog')
config.text('53 - Bluegrass Cellular')
config.text('54 - Boost')
config.text('55 - BPL mobile')
config.text('56 - Carolina Mobile Communications')
config.text('57 - Cellular One East Coast')
config.text('')
config.text('58 - Cellular One South West')
config.text('59 - Cellular One PCS')
config.text('60 - Cellular One')
config.text('61 - Cellular One West')
config.text('62 - Cellular South')
config.text('63 - Central Vermont Communications')
config.text('')
config.text('64 - CenturyTel')
config.text('65 - Chennai RPG Cellular')
config.text('66 - Chennai Skycell / Airtel')
config.text('67 - Cincinnati Bell')
config.text('68 - Cingular')
config.text('69 - Cingular Wireless')
config.text('')
config.text('70 - Clearnet')
config.text('71 - Comcast')
config.text('72 - Communication Specialists')
config.text('73 - Comviq')
config.text('74 - Cook Paging')
config.text('75 - Corr Wireless Communications')
config.text('')
config.text('215 - Cricket Wireless')
config.text('76 - Delhi Aritel')
config.text('77 - Digi-Page / Page Kansas')
config.text('78 - Dobson Cellular Systems')
config.text('79 - Dobson-Alex Wireless / Dobson-Cellular One')
config.text('80 - DT T-Mobile')
config.text('')
config.text('81 - Dutchtone / Orange-NL')
config.text('82 - Edge Wireless')
config.text('83 - EMT')
config.text('84 - Escotel')
config.text('85 - Fido')
config.text('86 - Galaxy Corporation')
config.text('')
config.text('87 - GCS Paging')
config.text('88 - Goa BPLMobil')
config.text('89 - Golden Telecom')
config.text('90 - GrayLink / Porta-Phone')
config.text('91 - GTE')
config.text('92 - Gujarat Celforce')
config.text('')
config.text('93 - Houston Cellular')
config.text('94 - Infopage Systems')
config.text('95 - Inland Cellular Telephone')
config.text('96 - JSM Tele-Page')
config.text('97 - Kerala Escotel')
config.text('98 - Kolkata Airtel')
config.text('')
config.text('99 - Kyivstar')
config.text('100 - Lauttamus Communication')
config.text('101 - LMT')
config.text('102 - Maharashtra BPL Mobile')
config.text('103 - Maharashtra Idea Cellular')
config.text('104 - Manitoba Telecom Systems')
config.text('')
config.text('105 - MCI Phone')
config.text('106 - MCI Page')
config.text('107 - Meteor')
config.text('108 - Metrocall')
config.text('109 - Metrocall 2-way')
config.text('110 - Metro PCS')
config.text('')
config.text('111 - Microcell')
config.text('112 - Midwest Wireless')
config.text('113 - MiWorld')
config.text('114 - Mobilecom PA')
config.text('115 - Mobilecomm')
config.text('116 - Mobileone')
config.text('')
config.text('117 - Mobilfone')
config.text('118 - Mobility Bermuda')
config.text('119 - Mobistar Belgium')
config.text('120 - Mobitel Tanzania')
config.text('121 - Mobtel Srbija')
config.text('122 - Morris Wireless')
config.text('')
config.text('123 - Motient')
config.text('124 - Movistar')
config.text('125 - Mumbai BPL Mobile')
config.text('126 - Mumbai Orange')
config.text('127 - NBTel')
config.text('128 - Netcom')
config.text('')
config.text('129 - NPI Wireless')
config.text('130 - Ntelos')
config.text('131 - O2')
config.text('132 - O2 (M-mail)')
config.text('133 - Omnipoint')
config.text('134 - One Connect Austria')
config.text('')
config.text('135 - OnlineBeep')
config.text('136 - Optus Mobile')
config.text('137 - Orange')
config.text('138 - Orange Mumbai')
config.text('139 - Orange - NL / Dutchtone')
config.text('140 - Oskar')
config.text('')
config.text('141 - P&T Luxembourg')
config.text('142 - Pacific Bell')
config.text('143 - PageMart')
config.text('144 - PageMart Advanced /2way')
config.text('145 - PageMart Canada')
config.text('')
config.text('146 - PageNet Canada')
config.text('147 - PageOne NorthWest')
config.text('148 - PCS One')
config.text('149 - Personal Communication')
config.text('150 - Pioneer / Enid Cellular')
config.text('151 - PlusGSM')
config.text('')
config.text('152 - Pondicherry BPL Mobile')
config.text('153 - Powertel')
config.text('154 - Price Communications')
config.text('155 - Primco')
config.text('156 - Primtel')
config.text('157 - ProPage')
config.text('')
config.text('158 - Public Service Cellular')
config.text('159 - Qualcomm')
config.text('160 - Qwest')
config.text('161 - RAM Page')
config.text('162 - Rogers AT&T Wireless')
config.text('')
config.text('163 - Rogers Canada')
config.text('164 - Safaricom')
config.text('165 - Satelindo GSM')
config.text('166 - Satellink')
config.text('167 - SBC Ameritech Paging')
config.text('168 - SCS-900')
config.text('')
config.text('169 - SFR France')
config.text('170 - Skytel Pagers')
config.text('171 - Simple Freedom')
config.text('172 - Smart Telecom')
config.text('173 - Southern LINC')
config.text('174 - Southwestern Bell')
config.text('')
config.text('175 - Sprint')
config.text('176 - Sprint PCS')
config.text('177 - ST Paging')
config.text('178 - SunCom')
config.text('179 - Sunrise Mobile')
config.text('180 - Surewest Communicaitons')
config.text('')
config.text('181 - Swisscom')
config.text('182 - T-Mobile / Voicestream')
config.text('183 - T-Mobile Austria')
config.text('184 - T-Mobile Germany')
config.text('185 - T-Mobile UK')
config.text('186 - Tamil Nadu BPL Mobile')
config.text('')
config.text('187 - Tele2 Latvia')
config.text('188 - Telefonica Movistar')
config.text('189 - Telenor')
config.text('190 - Teletouch')
config.text('191 - Telia Denmark')
config.text('192 - Telus')
config.text('')
config.text('193 - TIM')
config.text('194 - Triton')
config.text('195 - TSR Wireless')
config.text('196 - UMC')
config.text('197 - Unicel')
config.text('198 - Uraltel')
config.text('')
config.text('199 - US Cellular')
config.text('200 - US West')
config.text('201 - Uttar Pradesh Escotel')
config.text('202 - Verizon Pagers')
config.text('203 - Verizon PCS')
config.text('204 - Vessotel')
config.text('')
config.text('205 - Virgin Mobile')
config.text('206 - Vodafone Italy')
config.text('207 - Vodafone Japan')
config.text('208 - Vodafone Spain')
config.text('209 - Vodafone UK')
config.text('210 - WebLink Wiereless')
config.text('')
config.text('211 - West Central Wireless')
config.text('212 - Western Wireless')
config.text('213 - Wyndtell')
config.text('214 - Nextel')

config.write()


cvar_currentmap = es.ServerVar('eventscripts_currentmap')

###

ini = cfglib.AddonINI(os.path.join(es.getAddonPath(info.basename), info.basename + '_languages.ini'))

ini.setInitialComments(['# ./addons/eventscripts/%(basename)s/%(basename)s_languages.ini' % {'basename': info.basename},
 '# >>> For general addon configuration see ' + info.basename + '.cfg <<<', '', '# DO NOT translate words with $ or # in front!',
 '# For a list of language abbreviations please see ./addons/eventscripts/_libs/python/deflangs.ini'])

ini.addGroup('demand command')
ini.addValueToGroup('demand command', 'en', '#defaultSay#lightgreen  $name  #defaultto send a message to server admins.')

ini.addGroup('demand acknowledge')
ini.addValueToGroup('demand acknowledge', 'en', '#defaultYour message has been sent.')

ini.addGroup('demand noauth')
ini.addValueToGroup('demand noauth', 'en', '#defaultYou are not allowed access to the#lightgreen  $name  #defaultcommand.')

ini.addGroup('demand subject')
ini.addValueToGroup('demand subject', 'en', 'On-demand alert')

ini.addGroup('restart subject')
ini.addValueToGroup('restart subject', 'en', 'Server restarted')

ini.addGroup('restart text')
ini.addValueToGroup('restart text', 'en', 'Server restarted at $datetime')

ini.addGroup('cvar subject')
ini.addValueToGroup('cvar subject', 'en', 'cvar change alert')

ini.addGroup('cvar text')
ini.addValueToGroup('cvar text', 'en', 'cvar $var changed from "$old" to "$new".')

ini.addGroup('addban subject')
ini.addValueToGroup('addban subject', 'en', 'Server ban added')

ini.addGroup('addban text')
ini.addValueToGroup('addban text', 'en', '$name @ $identifier was $duration from the server.')

ini.addGroup('addban text noname')
ini.addValueToGroup('addban text noname', 'en', '$identifier was $duration from the server.')

ini.addGroup('addban permanent')
ini.addValueToGroup('addban permanent', 'en', 'permanently banned')

ini.addGroup('addban temporary')
ini.addValueToGroup('addban temporary', 'en', 'banned for $duration minutes')

ini.write()
lang_text = langlib.Strings(ini)


def removeTags(text):
   return text.replace('#green', '').replace('#lightgreen', '').replace('#default', '')

###

class SMS_Manager(threading.Thread):
   """ This threaded class handles sending SMS messages """

   def run(self):
      """ Contains all funcitonality to be executed in a separate thread """
      # We loop our SMS service until specifically stopped
      while True:
         # Retrieve information passed to the thread
         args = threading_q.get()
         # None is our cue to exit
         if args is None:
            break
         # Send the passed message
         self.sendTxtMsg(*args[0], **args[1])

   def sendTxtMsg(self, number, message, carrier, subject="Server alert", sender=cvar_return_address):
      """ Sends a text message to a cell phone """
      number = str(number)
      for text in self.msgSplit(message.replace('\\', '')):
         urllib.urlopen('http://www.onlinetextmessage.com/send.php',
          data=urllib.urlencode({'number': number,
                                 'message': text,
                                 'carrier': carrier,
                                 'from': str(sender),
                                 'subject': subject})).close()

   @staticmethod
   def msgSplit(message):
      """ Returns the message split into 100-character strings """
      # We return the result as a list
      result = []
      # Strip whitespace from the beginning and end of the message
      message = message.strip()
      # While our message is too long for a single message we need to cut it down
      while len(message) > MAX_MSG_LENGTH:
         # We find the first space before our message limit as our cutoff point
         space_location = message.rfind(' ', 1, MAX_MSG_LENGTH)
         # If no space was found we return the most characters we can
         if space_location > 0:
            # We add the characters truncated at the space to the result and remove them from our remaining message
            result.append(message[:space_location])
            message = message[space_location + 1:]
         else:
            # If no space was found we return the most characters we can
            result.append(message[:MAX_MSG_LENGTH])
            # We don't want to skip a character in this case because we couldn't find a space
            message = message[MAX_MSG_LENGTH:]
      # Append the remaining text to the result
      result.append(message)
      # Return the resul
      return result


threading_q = Queue() # Queue for passing information to the threads

def sendTxtMsg(*a, **kw):
   """ Sends arguments to the thread to be processed """
   threading_q.put((a, kw))


def startThreads():
   """ Start threads """
   for x in xrange(THREAD_COUNT):
      SMS_Manager().start()
startThreads()

def stopThreads():
   """ Stop running threads """
   for x in xrange(THREAD_COUNT):
      threading_q.put(None)

###

class AlertManager(object):
   class PhoneNumber(object):
      steamid = None

      def __init__(self, number, carrier):
         self.number  = number
         self.carrier = carrier
         self.alerts  = set()

   """ Begin AlertManager class """

   def __init__(self):
      self.numbers = {}

      # Phone number commands
      cmdlib.registerServerCommand('sms_number_add', self.addNumber, 'sms_number_add <phone number> <carrier> ["Steam ID"]\nAdds a phone number to SMS')
      cmdlib.registerServerCommand('sms_number_remove', self.removeNumber, 'sms_number_remove <phone number>\nRemoves a phone number from SMS')

      # Alert commands
      cmdlib.registerServerCommand('sms_register_for_alert', self.registerForAlert, 'sms_register_for_alert <phone number> <"alert name">\nRegisters a phone number to receive a certain alert')
      cmdlib.registerServerCommand('sms_unregister_for_alert', self.unregisterForAlert, 'sms_unregister_for_alert <phone number> <"alert name">\nUnregisters a phone number frim receiving a certain alert')

   @staticmethod
   def unregisterCommands():
      """ Unregisters server commands used by the alert manager """
      # Phone number commands
      cmdlib.unregisterServerCommand('sms_number_add')
      cmdlib.unregisterServerCommand('sms_number_remove')

      # Alert commands
      cmdlib.unregisterServerCommand('sms_register_for_alert')
      cmdlib.unregisterServerCommand('sms_unregister_for_alert')

   """ Phone number functions """

   def addNumber(self, args):
      """
      sms_number_add <phone number> <carrier> ["Steam ID"]
      """
      if len(args) in (2, 3):
         number = self.formatNumber(args[0])
         current = self.numbers[number] = self.PhoneNumber(number, int(args[1]))
         if len(args) == 3:
            current.steamid = args[2]

      else:
         es.dbgmsg(0, 'Syntax: sms_number_add <phone number> <carrier> ["Steam ID"]')

   def removeNumber(self, args):
      """
      sms_number_remove <phone number>
      """
      if len(args) == 1:
         number = self.formatNumber(args[0])
         if number in self.numbers:
            del self.numbers[number]

      else:
         es.dbgmsg(0, 'Syntax: sms_number_remove <phone number>')

   """ Alert functions """

   def registerForAlert(self, args):
      """
      sms_register_for_alert <phone number> <"alert name">
      """
      if len(args) == 2:
         number = self.formatNumber(args[0])
         if number not in self.numbers:
            raise ValueError, 'Phone number "%s" is not registered with SMS, cannot register for alert "%s"' % (number, args[1])
         self.numbers[number].alerts.add(args[1])

      else:
         es.dbgmsg(0, 'Syntax: sms_register_for_alert <phone number> <"alert name">')

   def unregisterForAlert(self, args):
      """
      sms_unregister_for_alert <phone number> <"alert name">
      """
      if len(args) == 2:
         number = self.formatNumber(args[0])
         if number in self.numbers:
            current = self.numbers[number]
            if args[2] in current.alerts:
               current.alerts.remove(args[2])

      else:
         es.dbgmsg(0, 'Syntax: sms_unregister_for_alert <phone number> <"alert name">')

   def sendAlert(self, name, subject, text):
      """ Sends an alert to all phone numbers registered to receive it """
      for number in self.numbers:
         current = self.numbers[number]
         if name in current.alerts:
            # ESC text
            if cvar_connected_display and not current.steamid is None and es.getuserid(current.steamid):
               es.esctextbox(cvar_connected_display, es.getuserid(current.steamid), subject, text)
            # Text message
            else:
               sendTxtMsg(current.number, text, current.carrier, subject)

   """ Miscellaneous functions """

   @staticmethod
   def formatNumber(number):
      return str(number).replace(' ', '').replace('-', '').replace('(', '').replace(')', '')

alerts = AlertManager()


def sendAlert(*a):
   alerts.sendAlert(*a)
sendAlert.__doc__ = AlertManager.sendAlert.__doc__

###

class DemandMessaging(object):
   command = None
   auth = None
   auth_levels = ('ROOT', 'ADMIN', 'POWERUSER', 'IDENTIFIED')

   def __init__(self):
      """ Register authorization for the on-demand aspect of SMS """
      # If auth is not a loaded service we cannot register a capability with it
      # In this case all known players will be allowed to use the on-deman command
      if not services.isRegistered('auth'):
         return
      self.auth = services.use('auth')
      # Convert the cvar to uppercase letters
      auth_level = str(cvar_demand_authorization).upper()
      # Find the integer permission for the de3sired level of authorization
      permission = self.auth.__getattribute__(auth_level if auth_level in self.auth_levels else 'IDENTIFIED')
      self.auth.registerCapability('sms_demand', permission)

   def createCommand(self, name):
      """ Create the on-demand SMS command """
      if not self.command is None:
         self.removeCommand()
      self.command = name
      cmdlib.registerSayCommand(name, self.demandCmd, 'Say  "' + name + '"  to send a text message to server admins')

   def removeCommand(self):
      """ Remove the on-demand SMS command """
      if not self.command is None:
         cmdlib.unregisterSayCommand(self.command)
      self.command = None

   def demandCmd(self, userid, args):
      """ Sends the provided text message designated admins """
      lang = playerlib.getPlayer(userid).lang

      # This player is not authorized to send an on-demand message
      if not self.isAuthed(userid):
         if cvar_demand_command_noauth_chat == 1:
            es.cexec(userid, 'echo ' + removeTags(lang_text('demand noauth', {'name': self.command}, lang)))
         elif cvar_demand_command_noauth_chat:
            es.tell(userid, '#multi', lang_text('demand noauth', {'name': self.command}, lang))

         if cvar_demand_command_noauth_center:
            es.centertell(userid, removeTags(lang_text('demand noauth', {'name': self.command}, lang)))

         if cvar_demand_command_noauth_hud:
            hudhint(userid, removeTags(lang_text('demand noauth', {'name': self.command}, lang)))

         return

      # Send the on-demand alert
      sendAlert('on_demand', lang_text('demand subject'), es.getargs())

      # Acknowledge the alert was sent
      if cvar_demand_command_acknowledge_chat == 1:
         es.cexec(userid, 'echo ' + removeTags(lang_text('demand acknowledge', lang=lang)))
      elif cvar_demand_command_acknowledge_chat:
         es.tell(userid, '#multi', lang_text('demand acknowledge', lang=lang))

      if cvar_demand_command_acknowledge_center:
         es.centertell(userid, removeTags(lang_text('demand acknowledge', lang=lang)))

      if cvar_demand_command_acknowledge_hud:
         hudhint(userid, removeTags(lang_text('demand acknowledge', lang=lang)))

   def displayCommand(self, userid):
      """ Displays to the player the on-demand command if the player is authorized """
      # No command, no display
      if not self.isAuthed(userid) or self.command is None:
         return

      # Tell the player the on-demand command
      lang = playerlib.getPlayer(userid).lang
      if cvar_demand_command_announce_chat == 1:
         es.cexec(userid, 'echo ' + removeTags(lang_text('demand command', {'name': self.command}, lang)))
      elif cvar_demand_command_announce_chat:
         es.tell(userid, '#multi', lang_text('demand command', {'name': self.command}, lang))

      if cvar_demand_command_announce_hud:
         hudhint(userid, removeTags(lang_text('demand command', {'name': self.command}, lang)))

   def isAuthed(self, userid):
      """ Returns True if the player is authorized to use the on-demand command otherwise returns False """
      # If there is no registered authorization service we only require a valid Steam ID
      if self.auth is None:
         if es.getplayersteamid(userid) != 'STEAM_ID_PENDING':
            # The Steam ID is not pending so we allow the player to use the command
            return True

      # With an auth service loaded we use it to determine whether the player is authorized
      return self.auth.isUseridAuthorized(userid, 'sms_demand')

on_demand = DemandMessaging()

###

class CvarAlerts(object):
   # Number of rounds to ignore cvar alerts
   # We want to ignore the first few rounds after startup to give the
   # server a chance to set the initial values of its variables
   rounds_until_active = 2

   def __init__(self):
      self.cvars = {}

      cmdlib.registerServerCommand('sms_cvar_watch', self.watchCvar, 'sms_cvar_watch <cvar>\nMarks a cvar for SMS to alert on changes')
      cmdlib.registerServerCommand('sms_cvar_unwatch', self.unwatchCvar, 'sms_cvar_unwatch <cvar>\nStops SMS watching a cvar')

   @staticmethod
   def unregisterCommands():
      """ Unregisters cvar watching commands """
      cmdlib.unregisterServerCommand('sms_cvar_watch')
      cmdlib.unregisterServerCommand('sms_cvar_unwatch')

   """ Server command functions """

   def watchCvar(self, args):
      """
      sms_cvar_watch <cvar>
      """
      if len(args) == 1:
         current = es.ServerVar(args[0])
         self.cvars[args[0]] = {'now': current, 'last': str(current)}

      else:
         es.dbgmsg(0, 'Syntax: sms_cvar_watch <cvar>')

   def unwatchCvar(self, args):
      """
      sms_cvar_unwatch <cvar>
      """
      if len(args) == 1:
         if args[0] in self.cvars:
            del self.cvars[args[0]]

      else:
         es.dbgmsg(0, 'Syntax: sms_cvar_unwatch <cvar>')

   """ Cvar watching functions """

   def checkVars(self):
      """ Checks all cvars for inconsistency """
      for name in self.cvars:
         self.cvarChange(name)

      # If we are not yet firing the cvar alerts we need to take one more step towards doing so
      if self.rounds_until_active:
         self.rounds_until_active -= 1

   def cvarChange(self, name):
      """ Checks a single cvar for inconsistency """
      # If we are not watching the cvar we don't care it has changed
      if name not in self.cvars:
         return

      # Check for changes and alert on change
      current = self.cvars[name]
      if current['now'] != current['last']:
         # If we are active we can fire the alert, otherewise we just update the new setting
         if not self.rounds_until_active:
            tokens = {'var': name, 'old': current['last'], 'new': current['now']}
            sendAlert('cvar_' + name, lang_text('cvar subject'), lang_text('cvar text', tokens))
         current['last'] = str(current['now'])

cvar_alerts = CvarAlerts()

###

def load():
   config.execute()
   on_demand.createCommand(cvar_demand_command)

   # If we loaded while a map is running we want to immediately have access to cvar alerts
   if str(cvar_currentmap):
      cvar_alerts.rounds_until_active = 0

   # No map means the addon was loaded in autoexec.cfg (or similar)
   else:
      sendAlert('server_restart', lang_text('restart subject'), lang_text('restart text', {'datetime': time.strftime('%X %x')}))


def round_start(event_var):
   cvar_alerts.checkVars()


def server_cvar(event_var):
   cvar_alerts.cvarChange(event_var['cvarname'])


def player_spawn(event_var):
   on_demand.displayCommand(int(event_var['userid']))


def server_addban(event_var):
   # "server_addban" alert
   name = event_var['name']
   steamid = event_var['networkid']
   duration = event_var['duration']
   sendAlert('server_addban', lang_text('addban subject'), lang_text('addban text' if name else 'addban text noname', 
    {'name': name,
     'identifier': event_var['ip'] if steamid in ('STEAM_ID_LAN', '') else steamid,
     'duration': lang_text('addban permanent' if duration == '0' else 'addban temporary',
      {'duration': duration}),
    }))


def unload():
   stopThreads()

   for cvar in config.getCvars():
      es.ServerVar(cvar).set(0)
   on_demand.removeCommand()
   alerts.unregisterCommands()
   cvar_alerts.unregisterCommands()